(function (Drupal, $) {
  Drupal.behaviors.dreamPermissions = {
    attach: function (context, settings) {
      $('body').once('dream-permissions', function () {
        var $buttonFetch = $('<button>Fetch permissions</button>');
        var $buttonSave = $('<button>Save permissions</button>');
        var $spinner = $('<div class="spinner">' + Drupal.t('Please wait') + '</div>');
        var $actions = $('.dream-permissions--form .form-actions');
        var $checks = $('.dream-permissions--checks');
        var token = '';
        var originalPermissions = {};

        // Show filters.
        $('.dream-permissions--form .no-js').removeClass('no-js');

        $actions.append($buttonFetch);
        $actions.append($buttonSave);
        $actions.append($spinner);
        $buttonSave.hide();
        $('.dream-permissions--form #edit-submit').remove();

        $('.dream-permissions--form .dream-permissions--filter').bind('input', function (event) {
          var $filter = $(this);
          var search = $filter.val().toLowerCase();
          $filter.closest('.fieldset-wrapper').find('.form-type-checkbox').each(function () {
            if (this.innerText.toLowerCase().indexOf(search) != -1) {
              this.classList.remove('unmatched');
            }
            else {
              this.classList.add('unmatched');
            }
          });
        });

        $buttonSave.bind('click', function (event) {
          event.preventDefault();
          $spinner.addClass('active');
          var data = {};
          $checks.find('input[type="checkbox"]').map(function () {
            // Split name in role and permission.
            var parts = this.name.match(/(\d*)\[(.*)\]/);
            if (typeof data[parts[1]] === "undefined") {
              data[parts[1]] = {};
            }
            data[parts[1]][parts[2]] = this.checked ? '1' : '0';
          });

          // Issue a post request.
          $.post(Drupal.settings.basePath + Drupal.settings.pathPrefix + 'admin/people/dream_permissions/save', {
            'token': token,
            'originalPermissions': originalPermissions,
            'permissions': data
          }, function() {
            $spinner.removeClass('active');
          })
        });

        $buttonFetch.bind('click', function (event) {
          event.preventDefault();
          $spinner.addClass('active');

          var selectedRoles = $('.dream-permissions--role-names input:checked').map(function () {
            return this.value;
          }).get().join(",");
          var selectedModules = $('.dream-permissions--module-names input:checked').map(function () {
            return this.value;
          }).get().join(",");

          if (selectedRoles.length > 0 && selectedModules.length > 0) {
            var url = Drupal.settings.basePath + Drupal.settings.pathPrefix + 'admin/people/dream_permissions/fetch/' + selectedModules + '/' + selectedRoles;
            var permission_filter = $('.dream-permissions--filter-permission').val();
            if (permission_filter) {
              url += '/' + permission_filter;
            }
            $.get(url, function (data) {

              // Store the token.
              token = data.token;

              // Collapse fieldsets.
              $('.dream-permissions--form fieldset').addClass('collapsed');

              // Clean output.
              $checks.empty();

              // Add filter for the rows.
              var $filterTable = $('<input class="form-text" type="text" placeholder="' + Drupal.t('Filter table rows') + '">');
              $filterTable.bind('input', function () {
                var $filter = $(this);
                var search = $filter.val().toLowerCase();
                $('#permissions-table tr').each(function () {
                  if (this.innerText.toLowerCase().indexOf(search) != -1) {
                    this.classList.remove('unmatched');
                  }
                  else {
                    this.classList.add('unmatched');
                  }
                });
              });
              $checks.append($filterTable);

              var $table = $('<table id="permissions-table"></table>');

              // Add header row.
              var roles = data.roles;
              var $row = $('<tr></tr>');
              $row.append('<th class="label">' + Drupal.t('Permission') + '</th>');
              for (var role in roles) {
                $row.append('<th class="label">' + roles[role] + '</th>');
              }
              $table.append($row);

              // Add checkbox rows.
              var permissions = data.permissions;
              originalPermissions = permissions;
              var permissionsNames = data.permissionsNames;

              for (var permissionName in permissions['2']) {
                var $row = $('<tr></tr>');
                $row.append('<td class="label">' + permissionsNames[permissionName] + '</td>');
                for (var rid in permissions) {
                  // Do not disable for anonymous user.
                  var disabled = rid !== '2' && rid !== '1' && permissions['2'][permissionName] === '1' ? ' disabled="disabled"' : '';
                  if (permissions[rid][permissionName] === '1') {
                    $row.append('<td class="item"><input type="checkbox" name="' + rid + '[' + permissionName + ']" value="1" checked="checked"' + disabled + '></td>');
                  }
                  else {
                    $row.append('<td class="item"><input type="checkbox" name="' + rid + '[' + permissionName + ']" value="1"' + disabled + '></td>');
                  }
                }
                $row.find('input[name="2[' + permissionName + ']"]').change(function () {
                  var name = $(this).attr('name');
                  name = '[' + name.replace(/^\d+\[/,'');
                  if ($(this).is(':checked')) {
                    $('#permissions-table').find('input[name$="' + name + '"]')
                      .not(this)
                      .not('input[name="1' + name + '"]')
                      .attr('disabled', 'disabled');
                  }
                  else {
                    $('#permissions-table').find('input[name$="' + name + '"]')
                      .not(this)
                      .not('input[name="1' + name + '"]')
                      .removeAttr('disabled');
                  }
                });
                $table.append($row);
              }

              $checks.append($table);
              $buttonSave.show();
              $spinner.removeClass('active');
            });
          }
        });
      });
    }
  };
})(Drupal, jQuery);
